      subroutine cmpt_intrps(hkl_max, interpolants,
     $     interpolants_size, t_analy, t_fftanaly, max_iq_slab,
     $     mem_cmpt, total_views , ctf_val, abs_ctf, algorithm)

      implicit none
      
      include 'mpif.h'
      include '../include/allprogs.inc'
      include '../include/info.inc'
      include '../include/com_01.inc'
      
C     Subroutine arguments
      integer, intent(in) :: hkl_max, max_iq_slab
      integer, intent(in) :: interpolants_size, algorithm
      real, intent(in) :: ctf_val(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col)
      real, intent(in) :: abs_ctf(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col)
      integer, intent(inout) :: total_views
      double precision, intent(out) :: mem_cmpt, t_analy, t_fftanaly

#ifdef SPREC
      real, intent(out) :: interpolants(interpolants_size)
#else
      double precision, intent(out) :: interpolants(interpolants_size)
#endif
      
      
C     Local variables
      integer view, symm, i, j, max_node_eq, max_count, datalen
      integer start_view, end_view, num_views, alloc_stat
      integer*8 clock1, clock2, clockmax, clockrate, ticks
      
      real du, dv, num_annulus, theta, phi, omega,
     +     center(2), imgmagfactor,
     +     rot_mat(3,3), ruv_mat_1(3,2,3), ruv_mat_2(3,2,3),
     +     ruv_sup_1(3,3), ruv_sup_2(3,3),
     +     theta_symm(60), phi_symm(60), omega_symm(60)

      real icos_rot(3,3,60)   ! icos rotation matrixes

C     Allocatable arrays
      real, allocatable ::  orient(:,:), rffta_omega(:)
      real, allocatable :: pixels_array(:,:,:) 
      real, allocatable ::  hkl_dft(:)
      complex, allocatable :: fc_2d(:,:)
      
      double precision :: mem_orient = 0.0D0,
     +     mem_rffta_omega = 0.0D0,
     +     mem_fc_2d = 0.0D0,
     +     mem_hkl_dft  = 0.0D0, mem_interp = 0.0D0,
     +     mem_interpmax = 0.0D0, mem_pixels_array = 0.0D0
      
      character(len=12) :: subname = 'cmpt_intrps'
      
      call system_clock(count_max=clockmax, count_rate=clockrate)
      
      if (algorithm == 1) then
         max_count = 0
      else if (algorithm == 3) then
         max_count = ((hkl_max+1)**2) * 2 * 3
      endif
      
      allocate(orient(0:9,max_views),
     +     rffta_omega(nrffta_omega),
     +     fc_2d(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col),
     +     pixels_array(npix_row, npix_col, view_perread),
     +     hkl_dft(max_count * 6),
     +     stat = alloc_stat)
      
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +        'Error in allocating memory for orient')
      endif
      
      mem_orient       = dble(4*10*max_views)
      mem_rffta_omega  = dble(4 * nrffta_omega)
      mem_fc_2d        = dble(8 * (1 + nfc_2d_row)*(1 + 2*nfc_2d_col))
      mem_pixels_array = dble(4 * view_perread * npix_row * npix_col)
      mem_hkl_dft      = dble(4 * max_count * 6)
      
      mem_cmpt = mem_orient + mem_rffta_omega + 
     $           mem_fc_2d  + mem_pixels_array + mem_hkl_dft
      
C     Initialize FFT workspace rffta_omega
      call vrffti( nrffta_col, rffta_omega )

      call system_clock(clock1)      
      num_views = 0 
      
      if(iam == 0) then
         call readorient(orient, max_views, ort_file_name, num_views,
     +        npix_row, npix_col, oddeven)
         
         write(*,1001) iam, ort_file_name, num_views, max_views
 1001    format(/' Node = ',i2,':  Read orientation file: ',a/,
     +        12x,' Number of views     = ', i9, i9)
      endif
      
C     Broadcast orientation info from root to all other processes
      if(num_nodes > 1) then
         call mpi_bcast(num_views, 1, mpi_integer,
     +        0, mpi_comm_world, mpi_err)
         if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +        'Error in calling MPI_BCAST num_views')
         
         call mpi_bcast(orient, 10*max_views, mpi_real,
     +        0, mpi_comm_world, mpi_err)
         if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +        'Error in calling MPI_BCAST orient')
      endif
      
      call system_clock(clock2)
      ticks = clock2-clock1
      if(ticks < 0) then
         ticks = clockmax + ticks
      endif
      t_analy = t_analy + dble(ticks)/dble(clockrate)
      
      total_views = total_views + num_views
      
C     calculate the radius in FFT space for 3D DFT
      num_annulus = real(nrffta_row)*PIXSIZ_OUT/res_max 
      if (num_annulus >= nrffta_row/2.0) then
         call error_stop(progname,subname,
     +        'Resolution is too high, should less than 2*PIXEL_SIZE')
      endif
      
C     we seperate all the views by view_perread, each time we read at
C     most view_perread * num_nodes pixels and deal with them
      
      
      do j = 0, (num_views-1)/(view_perread*num_nodes)
         call system_clock(clock1)
         
         start_view = 1 + j * view_perread*num_nodes
         end_view   = start_view + view_perread*num_nodes - 1
         if (end_view > num_views) end_view = num_views
         
C     the pix_file_name has been opened in aamain
C     Read the current end_view-start_view+1 pixel frames.
         
         call set_indices(num_nodes, start_view, end_view,
     +        frst_lst_view, max_node_eq )

C     processor 0 read pixels_array, then send to processor i
         do i = num_nodes - 1 , 0 , -1
            if (frst_lst_view(2, i) >= frst_lst_view(1, i)) then
               call read_files(pixels_array, frst_lst_view(1,i),
     +              frst_lst_view(2,i), i, orient)
            endif
         enddo
         
         call system_clock(clock2)
         ticks = clock2-clock1
         if(ticks < 0) then
            ticks = clockmax + ticks
         endif
         t_analy = t_analy + dble(ticks)/dble(clockrate)
         
         start_view = frst_lst_view(1,iam)
         end_view   = frst_lst_view(2,iam)
         datalen    = 0
        
         do view = start_view, end_view

C     Get the orientation angles and center of next pixel frame
            theta        = orient(1,view)
            phi          = orient(2,view)
            omega        = orient(3,view)
            center(1)    = orient(4,view)
            center(2)    = orient(5,view)
            imgmagfactor = orient(6,view)
            
C     Adjust the imgmagfactor according to its own pixel_size
            imgmagfactor = imgmagfactor * PIXSIZ_OUT / angstrom_per_pix
            
C     Calculate set of equivalent orientations based on application of
C     symmetry operations. Choices include icoshedral, eightfold,
C     dihedral, and asymmetric

            if(symm_code == 532) then
               call genrot(icos_rot)
               do i=1,60
                  call equivalent_view(theta, phi, omega, 
     $                 icos_rot(1,1,i), theta_symm(i),
     $                 phi_symm(i), omega_symm(i))
               enddo
            elseif (symm_code == 8) then
               call eight_symmetry(theta_symm, phi_symm, omega_symm,
     $              theta, phi, omega)
            elseif (symm_code == 5 .or. symm_code == 52 .or.
     +              symm_code == 2 .or. symm_code == 222 .or.
     +              symm_code == 3 .or. symm_code == 32 .or.
     +              symm_code == 6 .or. symm_code == 622 .or.
     +              symm_code == 7 .or. symm_code == 72 .or.
     +              symm_code == 8 .or. symm_code == 822) then
               call dihedral_symmetry(theta_symm, phi_symm, omega_symm,
     $              theta, phi, omega, symm_code, num_symm)
            elseif (symm_code == 1) then
               theta_symm(1) = theta
               phi_symm(1)   = phi
               omega_symm(1) = omega
            else
               call error_stop(progname, subname,
     +              'Error: unsupported symmetry')
            endif
            
            call system_clock(clock1)
            
C     Compute Discrete Fourier Transform of pixel values:
            call fft_2dfft( pixels_array(1, 1, view-start_view+1), 
     +           npix_row, npix_col, center,
     +           nrffta_omega, rffta_omega, nrffta_row, 
     +           nrffta_col, nfc_2d_row, 
     +           nfc_2d_col, fc_2d )
            du = center(1) - real(int(center(1)))
            dv = center(2) - real(int(center(2)))
            call shift_fft(fc_2d, nfc_2d_row, nfc_2d_col, du, dv)
            
C     Apply CTF correction
            IF(CTFMODE > 0 .and. CTFMODE <= 4) then
               fc_2d = fc_2d * ctf_val ! Fortran 90 array syntax
            endif
            
            call system_clock(clock2)
            ticks = clock2-clock1
            if(ticks < 0) then
               ticks = clockmax + ticks
            endif
            t_fftanaly = t_fftanaly + dble(ticks)/dble(clockrate)

C     Accumulate contributions to interpolants arrays from all of the
C     symmetry related views of the particle image
            
            do symm = 1, num_symm
               
C     Set the rotation matrices which transform the plane containing
C     the projection of the particle to the (X,Y,Z) `standard space'
C     and get the inverse rot_mat

               call crowther_to_matrix(theta_symm(symm),
     $              -phi_symm(symm), -omega_symm(symm), rot_mat)
               
C     Set the matrix which is used to determine the intersection of
C     grid line in the standard 3-d space with the rotated place
C     containing the Fourier coefficients of the pixels

Cji     call rottrans_1( rot_mat, ruv_mat_1, ruv_sup_1, -0.5 )
Cji     call rottrans_1( rot_mat, ruv_mat_2, ruv_sup_2,  0.5 )
               call rottrans_1( rot_mat, ruv_mat_1, ruv_sup_1, -0.999 )
               call rottrans_1( rot_mat, ruv_mat_2, ruv_sup_2,  0.999 )

               call matrixentries_slab( nfc_2d_row, nfc_2d_col, fc_2d,
     +              hkl_max, interpolants,
     +              hkl_dft, max_count, datalen,
     +              rot_mat, ruv_mat_1, ruv_mat_2, ruv_sup_1, ruv_sup_2,
     +              num_annulus, abs_ctf, imgmagfactor, mapmagfactor,
     +              algorithm)
               
               if (algorithm == 3) then
                  mem_interp = 0
                  call exch_intpol(hkl_dft, max_count, datalen,
     +                 interpolants, hkl_max, max_iq_slab, mem_interp)
                  mem_interpmax = max( mem_interpmax, mem_interp)
                  datalen = 0
               endif
               
            enddo
            
         enddo
                  
         if(algorithm == 3) then
            do view = 1, max_node_eq - (end_view - start_view + 1)
               do symm = 1, num_symm
                  mem_interp = 0
                  call exch_intpol(hkl_dft, max_count, datalen,
     +                 interpolants, hkl_max, max_iq_slab, mem_interp)
                  mem_interpmax = max( mem_interpmax, mem_interp)
                  datalen = 0
               enddo
            enddo
         endif
         
      enddo
      
      deallocate(orient, rffta_omega, fc_2d, pixels_array)
      
      return
      end subroutine cmpt_intrps
