
C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE LIST_CCS     *****                *
C  *                                                                          *
C  *    COMPUTES AND THEN LISTS CORRELATION COEFFICIENTS AS A FUNCTION OF     *
C  *     PARTICLE RADIUS AND RESOLUTION.                                      *
C  *                                                                          *
C  *    JANUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.        *
C  *    LAST MODIFIED  4-JUN-1998 BY TSB (Use BIN to get correct radii).      *
C  ****************************************************************************
C
      SUBROUTINE LIST_CCS (BIN, NCOL, NROW, IDIM1, IDIM2, NROT, NANNULI,
     $     IMG, MSK, R4_IMG, FFT1, FFT3, FFT4, FFT5, PRJ_ROT,
     $     PPRJ, PMAP, FLTPMAP, IMG_NUM, IMG_BKG, XCEN, YCEN, XCEN_MODEL
     $     ,YCEN_MODEL, RES_MIN, RES_MAX, ANN_LO
     $     ,ANN_HI, RADIUS, PIXRES_MAX, DEL_PIXRES, DEL_ANG, CC_AVG
     $     ,CC_RES_GLOBAL, CC_RAD_GLOBAL, verbose)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, IDIM1, IDIM2, NROT, NANNULI, IMG_NUM
      INTEGER*4 I, J, ANN_LO, ANN_HI, BIN, verbose
      real*4    IMG(NCOL,NROW), MSK(NCOL,NROW)
      real*4    IMG_BKG
      REAL*4    XCEN, YCEN, XCEN_MODEL, YCEN_MODEL
      REAL*4    CCRES(2,20), CCRAD(2,400), DEL_PIXRES, RES_MIN, RES_MAX
      REAL*4    CC_RES_GLOBAL(2,20), CC_AVG, CC_RAD_GLOBAL(2,400)
      REAL*4    RADIUS, DEL_ANG, SUMRSQ, PIXRES_MAX
      complex   FFT1(IDIM1,IDIM2), FFT3(IDIM1,IDIM2)
      complex   FFT4(IDIM1,IDIM2), FFT5(IDIM1,IDIM2)
      REAL*4    R4_IMG(NCOL,NROW)
      REAL*4    PRJ_ROT(NCOL,NROW), FLTPMAP(NROT,NANNULI)
      REAL*4    PMAP(NROT,NANNULI), PPRJ(NROT,NANNULI)

      MSK = IMG ! f90 array syntax
      CALL MAP_2DMASK (MSK, NCOL, NROW, ANN_LO, ANN_HI,
     1     XCEN, YCEN, IMG_BKG)
      CALL MAP_FFT_FILL (MSK, NCOL, NROW, FFT4, IDIM1, IDIM2, IMG_BKG)
      CALL FFT_2D (FFT4, IDIM1, IDIM2)

      MSK = PRJ_ROT ! f90 array syntax
      CALL MAP_2DMASK (MSK, NCOL, NROW, ANN_LO, ANN_HI,
     1     XCEN_MODEL, YCEN_MODEL, 0.0)
      CALL MAP_FFT_FILL (MSK, NCOL, NROW, FFT5, IDIM1, IDIM2, 0.0)
      CALL FFT_2D (FFT5, IDIM1, IDIM2)

      CALL PFTCC_RES (FFT1, FFT3, FFT4, FFT5, IDIM1, IDIM2, CCRES, 
     2     DEL_PIXRES)

      IF(RES_MIN > 0.0) THEN
         CALL FFT_HIP (FFT1, IDIM1, IDIM2, RES_MIN, 0.0, .TRUE.)
      ENDIF
      IF(RES_MAX > 0.0) THEN
         CALL FFT_LOP (FFT1, IDIM1, IDIM2, RES_MAX, 0.0, .TRUE.)
      ENDIF

      CALL FFT_2D_BACK(FFT1, IDIM1, IDIM2)
      CALL FFT_MAP_FILL(FFT1, IDIM1, IDIM2, R4_IMG, NCOL, NROW, 0.0)
      CALL MAP_POLAR (R4_IMG, NCOL, NROW, FLTPMAP, NROT, NANNULI, XCEN, 
     1     YCEN, 1.0, RADIUS, DEL_ANG)
      CALL MAP_POLAR (PRJ_ROT, NCOL, NROW, PPRJ, NROT, NANNULI, 
     1     XCEN_MODEL, YCEN_MODEL, 1.0, RADIUS, DEL_ANG)
      CALL CC_RADII (PMAP, FLTPMAP, PPRJ, NROT, NANNULI, ANN_HI, CCRAD,
     1     verbose)
C
      SUMRSQ = 0.0
      DO I = 1,ANN_HI+1
        CC_RAD_GLOBAL(1,I) = CC_RAD_GLOBAL(1,I) + CCRAD(1,I)
        CC_RAD_GLOBAL(2,I) = CC_RAD_GLOBAL(2,I) + CCRAD(2,I)
        IF(I.GE.ANN_LO) THEN
          SUMRSQ = SUMRSQ + (BIN*(I-1))**2
          CC_AVG = CC_AVG + CCRAD(2,I)*((BIN*(I-1))**2)
        ENDIF
      ENDDO
Ca  XDY wants to track all radii with verbose == 1 
      if(verbose .ge. 1) then
         DO I = ANN_HI+1, nannuli
            CC_RAD_GLOBAL(1,I) = CC_RAD_GLOBAL(1,I) + CCRAD(1,I)
            CC_RAD_GLOBAL(2,I) = CC_RAD_GLOBAL(2,I) + CCRAD(2,I)
         enddo
      endif
      IF(SUMRSQ.GT.0.0) THEN
         CC_AVG = CC_AVG/SUMRSQ
       ELSE
         CC_AVG = 0.0
      ENDIF
      J = NINT(1/(PIXRES_MAX*DEL_PIXRES))
      DO I = 1,J
        CC_RES_GLOBAL(1,I) = CC_RES_GLOBAL(1,I) + CCRES(1,I)
        CC_RES_GLOBAL(2,I) = CC_RES_GLOBAL(2,I) + CCRES(2,I)
      ENDDO
crss      WRITE (8,1000) IMG_NUM, (CCRES(1,I),I=1,J)
crss      WRITE(10,1000) IMG_NUM, (CCRES(2,I),I=1,J)
crss      WRITE (9,1010) ' CCs vs radius for particle', IMG_NUM
      DO I = 1,ANN_HI+1
        J = (I-1)*BIN
crss        WRITE (9,1020) J, CCRAD(1,I), CCRAD(2,I)
      ENDDO
crss      WRITE (9,1030) CC_AVG, ' = Average CC for ANN_LO,ANN_HI=', 
crss     1 ANN_LO, ANN_HI
C
 1000      FORMAT(I6,20F7.3)
 1010      FORMAT(A,I6)
 1020      FORMAT(I5,2F10.3)
 1030      FORMAT(F7.3,A,2I4)
      RETURN
      END
