      SUBROUTINE PFILE_INFO_PFT (MODE, NCOL, NROW, NSEC, NFILES, NIDS, 
     1     NIDS_TOT, PARAM_FNAMES, IMG_FNAMES, PIXSIZE)

C     This function had been renamed from PFILE_INFO so as to avoid
C     conflict with the subroutine of the same name in the Commpk
C     library developed by Yongchang Ji for the parallel codes. This
C     allows us to use Commpk without conflicts
C
C     READS IN ALL PARAMETER FILENAMES AND CHECKS THE CORRESPONDING
C     IMAGE FILES TO BE SURE THE IMAGE AND MODEL DATA HAVE THE SAME
C     DIMENSIONS.
C
C     Prompts user for a parameter data file, opens file, counts number
C     of particles, finds highest particle number, gets image filename
C     from first line of file.
C
C     Adapted from PFT_GETPFILE written by DMB, May 1995.             
C     
C     MODIFIED 19-JAN-1998 BY T. BAKER (Multiple inout files)  .        
C     MODIFIED 22-JAN-1998 BY T. BAKER (CTF parameters).                
C     MODIFIED  3-FEB-1998 BY T. BAKER (Save IMG_FILENAMES).            
C     MODIFIED  6-MAR-1998 BY TSB (Make UNIX compatible).               
C     MODIFIED  5-MAY-1998 BY TSB (Pass min pixel size to INFO).        
C     LAST MODIFIED 25-AUG-1999 BY TSB (Add UNITS variable).            
C     Modified 1/8/2007 by RSS - proper handling of blank lines,        
C     leading spaces in filenames, and pre-existing files               
C     plus some general cleanup and modernization                       

      implicit none

      integer, intent(in)  :: mode, ncol, nrow, nsec
      integer, intent(out) :: nfiles, nids(999), nids_tot
      character(len=256), intent(out) :: param_fnames(999)
      character(len=256), intent(out) :: img_fnames(999)
      real, intent(out) :: pixsize      

      INCLUDE '../include/pifhead.inc'
      TYPE (PIFHEAD_T) MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD

      COMMON/CTF_COM/PIXSIZ, UNITS, VOLTS, AMP_FAC, FOCUS_MAJ, 
     1     FOCUS_MIN, ANG_MAJ, CS_COEFF

      INTEGER NIMG, MAX_ID, ID, NCIMG, NRIMG, UNITS(999)
      integer ic, char
      REAL PIXSIZ(999), VOLTS(999), AMP_FAC(999)
      REAL FOCUS_MAJ(999), FOCUS_MIN(999), ANG_MAJ(999)
      REAL CS_COEFF(999)
      CHARACTER(len=256) NEW_FNAME, FNAME, JUNKSTR
      character(len=26) upper_case, lower_case
      CHARACTER READWRITE(4)
 
      data upper_case /'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
      data lower_case /'abcdefghijklmnopqrstuvwxyz'/

      NFILES   = 0              ! COUNT THE NUMBER OF PARAMETER FILES
      NIDS_TOT = 0              ! COUNT THE NUMBER OF IMAGES TO SEARCH
      PIXSIZE  = 1000000.0      ! Minimum pixel size 
      
C     Advance to proper part of file
      open(unit=17, file = 'stdin_temp')
 111  read(17,'(A)') junkstr
      do ic = 1, 3
         char = index(lower_case, junkstr(ic:ic))
         if (char .ne. 0) then
            junkstr(ic:ic) = upper_case(char:char)
         endif
      enddo
      if(junkstr(1:3) .ne. 'END') go to 111

      WRITE(*,'(A)') 'PARAMETER DATA FILES: '

 100  READ (17,'(A)',ERR=950,END=999) FNAME
      fname = adjustl(fname)
      if(len_trim(fname) == 0) go to 999
C     *************************************************************
C     *  Open file and read first line, make sure it is not data  *
C     *************************************************************
      IF(NFILES.NE.0) THEN
         CLOSE (UNIT=2)
         CALL PIF_CLOSE (7)
      ENDIF
      OPEN (UNIT=2, FILE=FNAME, STATUS='OLD', ERR=960, ACTION='READ')
      NFILES = NFILES + 1
      IF(NFILES.GT.999) GO TO 940
      READ (2,'(A)',ERR=970) IMG_FNAMES(NFILES)
      READ (2,*,ERR=975) PIXSIZ(NFILES), UNITS(NFILES), 
     1     VOLTS(NFILES), AMP_FAC(NFILES), FOCUS_MAJ(NFILES), 
     2     FOCUS_MIN(NFILES), ANG_MAJ(NFILES), CS_COEFF(NFILES)
      PARAM_FNAMES(NFILES) = FNAME
      CALL INCREMENT_FNAME (PARAM_FNAMES(NFILES), NEW_FNAME)
      IF(PIXSIZ(NFILES).LT.PIXSIZE) PIXSIZE = PIXSIZ(NFILES)
      PRINT *
      WRITE(*,'(A,I4,2A)') ' File #', NFILES, ' is: ',IMG_FNAMES(NFILES)
C     ************************************************************
C     *    OPEN UP PIF FORMAT IMAGE FILE AND CHECK TO MAKE SURE  *
C     *     MODEL MAP DATA SIZE AND IMAGE SIZE ARE EQUIVALENT    *
C     ************************************************************
      IMGHEAD%LUNUM = 7
      READWRITE(1)  = 'r'
      CALL PIF_OPEN (IMG_FNAMES(NFILES), IMGHEAD%LUNUM, READWRITE)
      CALL PIF_READ_GH (IMGHEAD)
      NIMG  = IMGHEAD%NIMG
      NCIMG = IMGHEAD%NUMCOLS
      NRIMG = IMGHEAD%NUMROWS
      IF(NCIMG.NE.NCOL .OR. NRIMG.NE.NROW) THEN
         PRINT *, '=================================================='
         PRINT *, '|  ERROR!!  Sizes of MAP and IMAGE are unequal.  |'
         PRINT *, '|  Make them equivalent then rerun program.      |'
         PRINT *, '=================================================='
         PRINT *, 'NCOL (map)   = ', NCOL
         PRINT *, 'NROW (map)   = ', NROW
         PRINT *, 'NSEC (map)   = ', NSEC
         PRINT *, 'NCOL (image) = ', NCIMG
         PRINT *, 'NROW (image) = ', NRIMG
         CALL TSBEND
      ENDIF
      IF(IABS(MODE).EQ.1) THEN
         NIDS(NFILES) = NIMG
         MAX_ID       = NIMG
         NIDS_TOT     = NIDS_TOT + NIMG
         GO TO 300
      ENDIF
C     ********************************************************************
C     *  Count particles in parameter file, find one with highest value  *
C     ********************************************************************
      NIDS(NFILES) = 0
      MAX_ID       = 0
 200  READ (2,*,ERR=980,END=300) ID !RSS
      NIDS_TOT     = NIDS_TOT + 1
      NIDS(NFILES) = NIDS(NFILES) + 1
      MAX_ID       = MAX(MAX_ID, ID)
      GO TO 200
C     ****************************************************
C     *    HERE WHEN AT END OF CURRENT PARAMETER FILE    *
C     ****************************************************
 300  IF(NIDS(NFILES).EQ.0)  GO TO 990
      GO TO 100                 ! CHECK FOR ADDITIONAL PARAMETER FILES


C     Error messsages
 940  PRINT *, '***********************************************'
      PRINT *, '* More than 999 parameter files: EXIT program *'
      CALL TSBEND
 950  PRINT *, '**************************************************'
      PRINT *, '* Error reading parameter filename: EXIT program *'
      CALL TSBEND
 960  PRINT *, '**********************************************'
      PRINT *, '* Error opening parameter file: EXIT program *'
      CALL TSBEND
 970  PRINT *, '**********************************************'
      PRINT *, '* Error reading IMAGE filename: EXIT program *'
      CALL TSBEND
 975  PRINT *, '**********************************************'
      PRINT *, '* READ ERROR: 2nd record of PARAMETER file   *'
      PRINT *, '* HINT: Did you specify UNITS?               *'
      CALL TSBEND
 980  PRINT *, '*******************************************'
      PRINT *, '* Error reading particle ID: EXIT program *'
      CALL TSBEND
 990  PRINT *, '*********************************************'
      PRINT *, '* PARAM file contains no data: EXIT program *'
      CALL TSBEND

 999  WRITE(*,'(I5,A,I6,A)') NFILES, ' PARAMETER files and total of', 
     1     NIDS_TOT, ' images for PFT search.'
      CLOSE (UNIT=2)
      CALL PIF_CLOSE (7)

      close(unit=17)

      return
      end subroutine pfile_info_pft
